@@ -23,13 +23,10 @@ along with GCC; see the file COPYING3.  If not see
 #include "tree.h"
 #include "cgraph.h"
 
-/* Used for parsing attributes of asm code.  */
-extern tree memory_identifier_string;
-
 struct ipa_dfs_info {
   int dfn_number;
   int low_link;
-  bool new;
+  bool new_node;
   bool on_stack;
   struct cgraph_node* next_cycle;
   PTR aux;
@@ -39,10 +36,11 @@ struct ipa_dfs_info {
 
 /* In ipa-utils.c  */
 void ipa_utils_print_order (FILE*, const char *, struct cgraph_node**, int);
-int ipa_utils_reduced_inorder (struct cgraph_node **, bool, bool);
+int ipa_utils_reduced_inorder (struct cgraph_node **, bool, bool,
+			       bool (*ignore_edge) (struct cgraph_edge *));
 tree get_base_var (tree);
 
- 
+
 #endif  /* GCC_IPA_UTILS_H  */
 
 
